==> 0.6.0.tar.gz: lib/api/ApiController.dart <==
import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'dart:typed_data';

import 'package:dio/adapter.dart';
import 'package:dio/adapter_browser.dart';
import 'package:dio/dio.dart';
import 'package:flutter/foundation.dart' show kIsWeb;
import 'package:flutter_secure_storage/flutter_secure_storage.dart' as storageSecure;
import 'package:http_parser/http_parser.dart';
import 'package:intl/intl.dart';
import 'package:mime/mime.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:snosey_flutter_package/Enums.dart';
import 'package:snosey_flutter_package/api/ApiResponse.dart';

class ApiController {
  static late String baseUrl;
  static String? appSecret;
  static String? apiKey;
  static String? userAgent;
  static late String serverErrorMessage;
  static String? refreshTokenUrl;
  static Map<String, String> staticParams = {};
  static String _cookies = "";
  final String apiUrl;
  final RequestTypeEnum requestType;
  static Map<String, String> staticHeaders = {
    "Access-Control-Allow-Origin": "*",
  };
  static ByteData? _pemCer;
  static ByteData? _keyCer;
  static String? _secretCer;
  Map<RequestTypeEnum, String> requestTypeMap = {
    RequestTypeEnum.PUT: "PUT",
    RequestTypeEnum.POST: "POST",
    RequestTypeEnum.GET: "GET",
    RequestTypeEnum.DELETE: "DELETE"
  };

  ApiController(this.apiUrl, this.requestType);

  static init({
    required String baseUrl,
    required String serverErrorMessage,
  }) {
    ApiController.baseUrl = baseUrl;
    ApiController.serverErrorMessage = serverErrorMessage;
  }

  static _addCookies(String key, String value) {
    _cookies += "$key=$value;";
  }

  Future<ApiResponse> sendRequest(
      {Map<String, dynamic> body = const {},
      Map<String, File> files = const {},
      bool isMultiPart = false,
      Map<String, String> header = const {},
      Map<String, String> queryParameters = const {}}) async {
    Map<String, String> finalQueryParams = {};
    finalQueryParams.addAll(queryParameters);
    finalQueryParams.addAll(staticParams);

    if (_cookies.isNotEmpty) staticHeaders["Cookie"] = _cookies;
    staticHeaders.addAll(header);
    var requestData;
    if (isMultiPart) {
      var requestData = FormData();
      if (files.isNotEmpty) {
        requestData.files.addAll(files.map((fileKey, file) {
          var mime = lookupMimeType(file.path);
          var fileByte = file.readAsBytesSync();
          var fileName = "${file.path.split('/').last}";
          var contentType = MediaType(mime!.split('/')[0], mime.split('/')[1]);
          return MapEntry(
              fileKey,
              MultipartFile.fromBytes(
                fileByte,
                filename: fileName,
                contentType: contentType,
              ));
        }).entries);
      }
      if (body.isNotEmpty) {
        body.forEach((key, value) {
          requestData.fields.add(fillMultiPart(key, value, "")!);
        });
      }
    } else {
      requestData = body;
    }
    var api = Dio(
      BaseOptions(
        baseUrl: baseUrl,
        contentType: 'application/json; charset=UTF-8',
        responseType: ResponseType.json,
        headers: staticHeaders,
        extra: {
          "withCredentials": true,
        },
        method: requestType.toString().replaceAll("RequestTypeEnum.", ""),
      ),
    );

    if (_pemCer != null) {
      if (!kIsWeb) {
        (api.httpClientAdapter as DefaultHttpClientAdapter).onHttpClientCreate =
            (client) {
          SecurityContext serverContext = new SecurityContext()
            ..useCertificateChainBytes(_pemCer!.buffer.asUint8List())
            ..usePrivateKeyBytes(_keyCer!.buffer.asUint8List(),
                password: _secretCer!);

          HttpClient httpClient = HttpClient(
            context: serverContext,
          );

          httpClient.badCertificateCallback =
              (X509Certificate cert, String host, int port) => true;

          return httpClient;
        };
      } else {
        (api.httpClientAdapter as BrowserHttpClientAdapter).withCredentials =
            true;
      }
    }

    api.interceptors.add(
      LogInterceptor(
        responseBody: true,
        error: true,
        request: true,
        requestBody: true,
        requestHeader: true,
        responseHeader: true,
      ),
    );
    var response = await api
        .request(
          apiUrl,
          data: requestData,
          options: Options(
            extra: {
              "withCredentials": true,
            },
          ),
          queryParameters: finalQueryParams,
        )
        .onError((error, stackTrace) => throw serverErrorMessage);
    if (response.statusCode == HttpStatus.ok) {
      if (response.headers.value("authorization") != null) {
        Auth._setBearerToken(
            token: response.headers.value("authorization")!,
            expiration: response.headers.value("expires")!);
        response.headers["set-cookie"]!.forEach((cookie) async {
          await Auth._setRefreshToken(cookie);
        });
      }

      var apiResponse = ApiResponse();
      apiResponse.setStatus(jsonDecode(response.headers.value("x-status")!));
      if (response.headers.value("x-pagination") != null) {
        apiResponse.pagination.hasPagination = true;
        apiResponse.pagination
            .set(jsonDecode(response.headers.value("x-pagination")!));
      } else {
        apiResponse.pagination.hasPagination = false;
      }
      if (apiResponse.success) {
        apiResponse.objectResponse = response.data;
        return apiResponse;
      } else
        throw utf8.decode(base64.decode(apiResponse.errorMessage));
    } else {
      throw serverErrorMessage;
    }
  }

  MapEntry<String, String>? fillMultiPart(
      String key, dynamic value, String root) {
    if (value is Map) {
      for (var object in value.entries) {
        return fillMultiPart(object.key, object.value, key + ".");
      }
    } else if (value is List) {
      for (int i = 0; i < value.length; i++)
        return fillMultiPart("$key[$i]", value[i], root);
    } else {
      return MapEntry("$root$key", value.toString());
    }
  }
}

class Auth {
  static var _expirationFormat = DateFormat("E, dd MMM yyy HH:mm:ss 'GMT'");
  static Timer? _reNewExpirationTimer;

  static init({
    required String appSecret,
    required UserAgentEnum userAgent,
    required String apiKey,
    required String refreshTokenUrl,
    ByteData? pemCer,
    ByteData? keyCer,
    required String? secretCer,
  }) {
    ApiController.refreshTokenUrl = refreshTokenUrl;
    ApiController._addCookies("Secret", appSecret);
    ApiController._addCookies("Api-Key", apiKey);
    ApiController.staticHeaders["User-Agent"] =
        userAgent.toString().replaceFirst("UserAgentEnum.", "");
    ApiController._pemCer = pemCer;
    ApiController._keyCer = keyCer;
    ApiController._secretCer = secretCer;
  }

  static _setBearerToken({
    required String token,
    required String expiration,
  }) {
    ApiController.staticHeaders["Authorization"] = token;
    var expirationMin = DateTime.now()
            .difference(_expirationFormat.parse(expiration))
            .inMinutes -
        5;
    print("renew in $expirationMin min");
    _reNewExpirationTimer = Timer(Duration(minutes: expirationMin), () {
      reNewToken();
    });
  }

  static Future<bool> reNewToken() async {
    String? expires;
    String? token;
    if (!kIsWeb) {
      final storage = new storageSecure.FlutterSecureStorage();
      expires = await storage.read(key: "expires");
      token = await storage.read(key: "refreshToken");
    } else {
      SharedPreferences prefs = await SharedPreferences.getInstance();
      expires = prefs.getString("expires");
      token = prefs.getString("refreshToken");
    }
    if (expires != null &&
        _expirationFormat.parse(expires).isAfter(DateTime.now())) {
      bool isAuth = false;

      await ApiController(ApiController.refreshTokenUrl!, RequestTypeEnum.POST)
          .sendRequest(body: {"Token": token!}).then((value) {
        isAuth = true;
      }).onError((error, stackTrace) {
        isAuth = false;
      });
      return isAuth;
    } else {
      return false;
    }
  }

  static clearToken() {
    if (_reNewExpirationTimer != null) _reNewExpirationTimer!.cancel();
    if (ApiController.staticHeaders.containsKey("Authorization"))
      ApiController.staticHeaders.remove("Authorization");

    if (!kIsWeb) {
      final storage = new storageSecure.FlutterSecureStorage();
      storage.deleteAll();
    } else {
      SharedPreferences.getInstance().then((value) => value.clear());
    }
  }

  static _setRefreshToken(String cookies) async {
    final storage = new storageSecure.FlutterSecureStorage();
    SharedPreferences prefs = await SharedPreferences.getInstance();
    cookies.split("; ").forEach((element) {
      var key = element.split("=").first;
      var value = element.split("=").last;
      if (key == "refreshToken" || key == "expires") {
        if (!kIsWeb)
          storage.write(key: key, value: value);
        else {
          prefs.setString(key, value);
        }
      }
    });
  }
}

==> 0.6.1.tar.gz: lib/api/ApiController.dart <==
import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'dart:typed_data';

import 'package:dio/adapter.dart';
import 'package:dio/adapter_browser.dart';
import 'package:dio/dio.dart';
import 'package:flutter/foundation.dart' show kIsWeb;
import 'package:flutter_secure_storage/flutter_secure_storage.dart'
    as storageSecure;
import 'package:http_parser/http_parser.dart';
import 'package:intl/intl.dart';
import 'package:mime/mime.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:snosey_flutter_package/Enums.dart';
import 'package:snosey_flutter_package/api/ApiResponse.dart';

class ApiController {
  static late String baseUrl;
  static String? appSecret;
  static String? apiKey;
  static String? userAgent;
  static late String serverErrorMessage;
  static String? refreshTokenUrl;
  static Map<String, String> staticParams = {};
  static String _cookies = "";
  final String apiUrl;
  final RequestTypeEnum requestType;
  static Map<String, String> staticHeaders = {};
  static ByteData? _pemCer;
  static ByteData? _keyCer;
  static String? _secretCer;
  Map<RequestTypeEnum, String> requestTypeMap = {
    RequestTypeEnum.PUT: "PUT",
    RequestTypeEnum.POST: "POST",
    RequestTypeEnum.GET: "GET",
    RequestTypeEnum.DELETE: "DELETE"
  };

  ApiController(this.apiUrl, this.requestType);

  static init({
    required String baseUrl,
    required String serverErrorMessage,
  }) {
    ApiController.baseUrl = baseUrl;
    ApiController.serverErrorMessage = serverErrorMessage;
  }

  static _addCookies(String key, String value) {
    _cookies += "$key=$value;";
  }

  Future<ApiResponse> sendRequest(
      {Map<String, dynamic> body = const {},
      Map<String, File> files = const {},
      bool isMultiPart = false,
      Map<String, String> header = const {},
      Map<String, String> queryParameters = const {}}) async {
    Map<String, String> finalQueryParams = {};
    finalQueryParams.addAll(queryParameters);
    finalQueryParams.addAll(staticParams);

    if (_cookies.isNotEmpty) staticHeaders["Cookie"] = _cookies;
    staticHeaders.addAll(header);
    var requestData;
    if (isMultiPart) {
      var requestData = FormData();
      if (files.isNotEmpty) {
        requestData.files.addAll(files.map((fileKey, file) {
          var mime = lookupMimeType(file.path);
          var fileByte = file.readAsBytesSync();
          var fileName = "${file.path.split('/').last}";
          var contentType = MediaType(mime!.split('/')[0], mime.split('/')[1]);
          return MapEntry(
              fileKey,
              MultipartFile.fromBytes(
                fileByte,
                filename: fileName,
                contentType: contentType,
              ));
        }).entries);
      }
      if (body.isNotEmpty) {
        body.forEach((key, value) {
          requestData.fields.add(fillMultiPart(key, value, "")!);
        });
      }
    } else {
      requestData = body;
    }
    var api = Dio(
      BaseOptions(
        baseUrl: baseUrl,
        contentType: 'application/json; charset=UTF-8',
        responseType: ResponseType.json,
        headers: staticHeaders,
        extra: {
          "withCredentials": true,
        },
        method: requestType.toString().replaceAll("RequestTypeEnum.", ""),
      ),
    );

    if (_pemCer != null) {
      if (!kIsWeb) {
        (api.httpClientAdapter as DefaultHttpClientAdapter).onHttpClientCreate =
            (client) {
          SecurityContext serverContext = new SecurityContext()
            ..useCertificateChainBytes(_pemCer!.buffer.asUint8List())
            ..usePrivateKeyBytes(_keyCer!.buffer.asUint8List(),
                password: _secretCer!);

          HttpClient httpClient = HttpClient(
            context: serverContext,
          );

          httpClient.badCertificateCallback =
              (X509Certificate cert, String host, int port) => true;

          return httpClient;
        };
      } else {
        (api.httpClientAdapter as BrowserHttpClientAdapter).withCredentials =
            true;
      }
    }

    api.interceptors.add(
      LogInterceptor(
        responseBody: true,
        error: true,
        request: true,
        requestBody: true,
        requestHeader: true,
        responseHeader: true,
      ),
    );
    var response = await api
        .request(
          apiUrl,
          data: requestData,
          options: Options(
            extra: {
              "withCredentials": true,
            },
          ),
          queryParameters: finalQueryParams,
        )
        .onError((error, stackTrace) => throw serverErrorMessage);
    if (response.statusCode == HttpStatus.ok) {
      if (response.headers.value("authorization") != null) {
        Auth._setBearerToken(
            token: response.headers.value("authorization")!,
            expiration: response.headers.value("expires")!);
        response.headers["set-cookie"]!.forEach((cookie) async {
          await Auth._setRefreshToken(cookie);
        });
      }

      var apiResponse = ApiResponse();
      apiResponse.setStatus(jsonDecode(response.headers.value("x-status")!));
      if (response.headers.value("x-pagination") != null) {
        apiResponse.pagination.hasPagination = true;
        apiResponse.pagination
            .set(jsonDecode(response.headers.value("x-pagination")!));
      } else {
        apiResponse.pagination.hasPagination = false;
      }
      if (apiResponse.success) {
        apiResponse.objectResponse = response.data;
        return apiResponse;
      } else
        throw utf8.decode(base64.decode(apiResponse.errorMessage));
    } else {
      throw serverErrorMessage;
    }
  }

  MapEntry<String, String>? fillMultiPart(
      String key, dynamic value, String root) {
    if (value is Map) {
      for (var object in value.entries) {
        return fillMultiPart(object.key, object.value, key + ".");
      }
    } else if (value is List) {
      for (int i = 0; i < value.length; i++)
        return fillMultiPart("$key[$i]", value[i], root);
    } else {
      return MapEntry("$root$key", value.toString());
    }
  }
}

class Auth {
  static var _expirationFormat = DateFormat("E, dd MMM yyy HH:mm:ss 'GMT'");
  static Timer? _reNewExpirationTimer;

  static init({
    required String appSecret,
    required UserAgentEnum userAgent,
    required String apiKey,
    required String refreshTokenUrl,
    ByteData? pemCer,
    ByteData? keyCer,
    required String? secretCer,
  }) {
    ApiController.refreshTokenUrl = refreshTokenUrl;
    ApiController.staticHeaders["Secret"]= appSecret;
    ApiController.staticHeaders["Api-Key"]= apiKey;
    ApiController.staticHeaders["User-Platform"] =
        userAgent.toString().replaceFirst("UserAgentEnum.", "");
    ApiController._pemCer = pemCer;
    ApiController._keyCer = keyCer;
    ApiController._secretCer = secretCer;
  }

  static _setBearerToken({
    required String token,
    required String expiration,
  }) {
    ApiController.staticHeaders["Authorization"] = token;
    var expirationMin = DateTime.now()
            .difference(_expirationFormat.parse(expiration))
            .inMinutes -
        5;
    print("renew in $expirationMin min");
    _reNewExpirationTimer = Timer(Duration(minutes: expirationMin), () {
      reNewToken();
    });
  }

  static Future<bool> reNewToken() async {
    String? expires;
    String? token;
    if (!kIsWeb) {
      final storage = new storageSecure.FlutterSecureStorage();
      expires = await storage.read(key: "expires");
      token = await storage.read(key: "refreshToken");
    } else {
      SharedPreferences prefs = await SharedPreferences.getInstance();
      expires = prefs.getString("expires");
      token = prefs.getString("refreshToken");
    }
    if (expires != null &&
        _expirationFormat.parse(expires).isAfter(DateTime.now())) {
      bool isAuth = false;

      await ApiController(ApiController.refreshTokenUrl!, RequestTypeEnum.POST)
          .sendRequest(body: {"Token": token!}).then((value) {
        isAuth = true;
      }).onError((error, stackTrace) {
        isAuth = false;
      });
      return isAuth;
    } else {
      return false;
    }
  }

  static clearToken() {
    if (_reNewExpirationTimer != null) _reNewExpirationTimer!.cancel();
    if (ApiController.staticHeaders.containsKey("Authorization"))
      ApiController.staticHeaders.remove("Authorization");

    if (!kIsWeb) {
      final storage = new storageSecure.FlutterSecureStorage();
      storage.deleteAll();
    } else {
      SharedPreferences.getInstance().then((value) => value.clear());
    }
  }

  static _setRefreshToken(String cookies) async {
    final storage = new storageSecure.FlutterSecureStorage();
    SharedPreferences prefs = await SharedPreferences.getInstance();
    cookies.split("; ").forEach((element) {
      var key = element.split("=").first;
      var value = element.split("=").last;
      if (key == "refreshToken" || key == "expires") {
        if (!kIsWeb)
          storage.write(key: key, value: value);
        else {
          prefs.setString(key, value);
        }
      }
    });
  }
}
